"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t;
    return { next: verb(0), "throw": verb(1), "return": verb(2) };
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var fs_extra_promise_1 = require("fs-extra-promise");
var debug = require('debug')('build:runner');
var Downloader_1 = require("./Downloader");
var FFmpegDownloader_1 = require("./FFmpegDownloader");
var BuildConfig_1 = require("./BuildConfig");
var archive_1 = require("./archive");
var util_1 = require("./util");
var Runner = (function () {
    function Runner(options, args) {
        if (options === void 0) { options = {}; }
        this.args = args;
        this.options = util_1.mergeOptions(Runner.DEFAULT_OPTIONS, options);
        debug('in constructor', 'args', args);
        debug('in constructor', 'options', this.options);
    }
    Runner.prototype.run = function () {
        return __awaiter(this, void 0, void 0, function () {
            var platform, arch, configPath, pkg, config, downloader, archivePath, _a, runtimeDir, cleanup, executable, _b, code, signal;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        platform = process.platform;
                        arch = this.options.x86 || this.options.x64
                            ? (this.options.x86 ? 'ia32' : 'x64')
                            : process.arch;
                        configPath = this.options.config ? this.options.config : path_1.join(this.args[0], 'package.json');
                        return [4 /*yield*/, fs_extra_promise_1.readJsonAsync(configPath)];
                    case 1:
                        pkg = _c.sent();
                        config = new BuildConfig_1.BuildConfig(pkg);
                        debug('in run', 'configPath', configPath);
                        debug('in run', 'config', config);
                        downloader = new Downloader_1.Downloader({
                            platform: platform, arch: arch,
                            version: config.nwVersion,
                            flavor: 'sdk',
                            mirror: this.options.mirror,
                            useCaches: true,
                            showProgress: this.options.mute ? false : true,
                        });
                        if (!this.options.mute) {
                            console.info('Fetching NW.js binary...', {
                                platform: downloader.options.platform,
                                arch: downloader.options.arch,
                                version: downloader.options.version,
                                flavor: downloader.options.flavor,
                            });
                        }
                        return [4 /*yield*/, downloader.fetch()];
                    case 2:
                        archivePath = _c.sent();
                        return [4 /*yield*/, util_1.tmpDir()];
                    case 3:
                        _a = _c.sent(), runtimeDir = _a.path, cleanup = _a.cleanup;
                        if (!this.options.mute) {
                            console.info('Extracting NW.js binary...', {
                                runtimeDir: runtimeDir,
                            });
                        }
                        if (!archivePath.endsWith('.zip')) return [3 /*break*/, 5];
                        return [4 /*yield*/, archive_1.extract(archivePath, runtimeDir)];
                    case 4:
                        _c.sent();
                        return [3 /*break*/, 8];
                    case 5:
                        if (!archivePath.endsWith('tar.gz')) return [3 /*break*/, 7];
                        return [4 /*yield*/, archive_1.extractTarGz(archivePath, runtimeDir)];
                    case 6:
                        _c.sent();
                        return [3 /*break*/, 8];
                    case 7: throw new Error('ERROR_UNKNOWN_EXTENSION');
                    case 8:
                        if (!config.ffmpegIntegration) return [3 /*break*/, 10];
                        return [4 /*yield*/, this.integrateFFmpeg(platform, arch, runtimeDir, pkg, config)];
                    case 9:
                        _c.sent();
                        _c.label = 10;
                    case 10: return [4 /*yield*/, util_1.findExecutable(platform, runtimeDir)];
                    case 11:
                        executable = _c.sent();
                        return [4 /*yield*/, fs_extra_promise_1.chmodAsync(executable, 365)];
                    case 12:
                        _c.sent();
                        if (!this.options.mute) {
                            console.info('Launching NW.js app...');
                        }
                        return [4 /*yield*/, util_1.spawnAsync(executable, this.args, {
                                detached: this.options.detached,
                            })];
                    case 13:
                        _b = _c.sent(), code = _b.code, signal = _b.signal;
                        cleanup();
                        if (!!this.options.mute) return [3 /*break*/, 16];
                        if (!this.options.detached) return [3 /*break*/, 15];
                        console.info('NW.js app detached.');
                        return [4 /*yield*/, new Promise(function (resolve, reject) {
                                setTimeout(resolve, 3000);
                            })];
                    case 14:
                        _c.sent();
                        return [3 /*break*/, 16];
                    case 15:
                        console.info("NW.js app exited with " + code + ".");
                        _c.label = 16;
                    case 16: return [2 /*return*/, code];
                }
            });
        });
    };
    Runner.prototype.integrateFFmpeg = function (platform, arch, runtimeDir, pkg, config) {
        return __awaiter(this, void 0, void 0, function () {
            var downloader, path, _a, ffmpegDir, cleanup, src, dest;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        downloader = new FFmpegDownloader_1.FFmpegDownloader({
                            platform: platform, arch: arch,
                            version: config.nwVersion,
                            useCaches: true,
                            showProgress: this.options.mute ? false : true,
                        });
                        if (!this.options.mute) {
                            console.info('Fetching FFmpeg prebuilt...', {
                                platform: downloader.options.platform,
                                arch: downloader.options.arch,
                                version: downloader.options.version,
                            });
                        }
                        return [4 /*yield*/, downloader.fetch()];
                    case 1:
                        path = _b.sent();
                        return [4 /*yield*/, util_1.tmpDir()];
                    case 2:
                        _a = _b.sent(), ffmpegDir = _a.path, cleanup = _a.cleanup;
                        if (!this.options.mute) {
                            console.info('Extracting FFmpeg prebuilt...', {
                                ffmpegDir: ffmpegDir,
                            });
                        }
                        if (!path.endsWith('.zip')) return [3 /*break*/, 4];
                        return [4 /*yield*/, archive_1.extract(path, ffmpegDir)];
                    case 3:
                        _b.sent();
                        return [3 /*break*/, 5];
                    case 4: throw new Error('ERROR_UNKNOWN_EXTENSION');
                    case 5: return [4 /*yield*/, util_1.findFFmpeg(platform, ffmpegDir)];
                    case 6:
                        src = _b.sent();
                        return [4 /*yield*/, util_1.findFFmpeg(platform, runtimeDir)];
                    case 7:
                        dest = _b.sent();
                        return [4 /*yield*/, fs_extra_promise_1.copyAsync(src, dest)];
                    case 8:
                        _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    return Runner;
}());
Runner.DEFAULT_OPTIONS = {
    x86: false,
    x64: false,
    mirror: Downloader_1.Downloader.DEFAULT_OPTIONS.mirror,
    detached: false,
    config: undefined,
    mute: true,
};
exports.Runner = Runner;
//# sourceMappingURL=Runner.js.map